from pyexternal_import.resource import BaseResource
from pyexternal_import.utils import flatten_serializer, rebuild_nested_data


class PlainExcelResource(BaseResource):
    """
    适合表头比较整齐的 excel
    """
    def validate_raw_df_data(self, df):
        return df

    def flat_df_to_dict_by_serializer(self, flat_df):
        """
        :param flat_df: 整理好的 dataframe，column name 为 serializer 的 label。
        :return: 根据 serializer 的结构，返回整理好的 dict 数据，可能是 nested 的结构。一个 object 对应一个 instance。
        """
        field_label = flatten_serializer(self.Meta.serializer_class._declared_fields)
        raw_data = []
        for index, row in flat_df.iterrows():
            flat_raw_data = {field: row.get(label) for field, label in field_label.items()}
            nested_raw_data = rebuild_nested_data(flat_raw_data)
            raw_data.append(nested_raw_data)
        return raw_data

    def get_info_from_plain_excel(self, file_obj, **kwargs):
        """
        从 plain excel 中获取清洗好的 raw_data
        :param file_obj:
        :param kwargs:
        :return:
        """
        df = self.get_raw_df_data_from_file(file_obj)
        df = self.validate_raw_df_data(df)
        raw_data = self.flat_df_to_dict_by_serializer(df)
        return raw_data
