from rest_framework.exceptions import ValidationError

from pyexternal_import.resource import BaseResource, RowResult
from pyexternal_import.serializer import TeamBitionSerializer
from pyexternal_import.utils import flatten_serializer, filter_none, rebuild_nested_data


class TeamBitionResource(BaseResource):

    def __init__(self, client):
        # client 为 djnago_teambition 的 client
        self.client = client

    def get_info_from_raw_list_data(self, raw_data_list, **kwargs):
        ignore_status = kwargs.pop('ignore_status', None)
        ready_for_deserialize = []
        for raw_data in raw_data_list:
            try:
                tb_status = self.client.project_info['id_task_flow_pair'][raw_data['_taskflowstatusId']]
                if tb_status == ignore_status:
                    continue
                creator = self.client.project_info['id_member_pair'][raw_data['_creatorId']]
                executor = self.client.project_info['id_member_pair'].get(raw_data['_executorId'])
                tags = self.get_tags(raw_data['tagIds'])
                teambition_id = raw_data['_id']
                content = raw_data['content']
                note = raw_data['note']
                history = raw_data['task_history']
                attachments = raw_data['task_attachments']
                other_fields = self.build_custom_fields(raw_data['customfields'])
            except KeyError:
                raise ValidationError('序列化 tb 任务时出错.')
            raw_data_dict = {'creator': creator, 'executor': executor, 'tb_status': tb_status, 'content': content,
                             'teambition_id': teambition_id, 'tags': tags, 'note': note, 'tb_history': history,
                             'tb_attachments': attachments, **other_fields}
            ready_for_deserialize.append(raw_data_dict)
        return ready_for_deserialize

    def substitute_chinese_key(self, row_chinese_custom_fields):
        """
        用 serializer 里的中文 key 的对应关系替换，row_chinese_custom_fields，返回替换好的英语 key
        :param row_chinese_custom_fields:
        :return:
        """
        field_label = flatten_serializer(self.Meta.serializer_class._declared_fields)
        field_label = filter_none(field_label)
        flat_raw_data = {field: row_chinese_custom_fields.get(label) for field, label in field_label.items()}
        nested_raw_data = rebuild_nested_data(flat_raw_data)
        return nested_raw_data

    def get_tags(self, tag_id_list):
        if not tag_id_list:
            return None
        else:
            return [self.client.project_info['id_tag_pair'][tag_id] for tag_id in tag_id_list]

    def build_custom_fields(self, custom_fields):
        other_fields_chinese_key = {}
        for field in custom_fields:
            try:  # such as other_fields['基金简称']
                other_fields_chinese_key[
                    self.client.project_info['custom_id_field_pair'][field['_customfieldId']]] = \
                    field['value'][0]['title']
            # field['value'][0]
            except IndexError:
                other_fields_chinese_key[
                    self.client.project_info['custom_id_field_pair'][field['_customfieldId']]] = None
            # 增改字段后可能有些任务 tb 的数据可能没有及时清理 导致出现了不显示在 tb 系统上的值
            # self.client.project_info['custom_id_field_pair'][field['_customfieldId']
            except KeyError:
                continue
        return self.substitute_chinese_key(other_fields_chinese_key)

    def get_context(self):
        return self.client

    def get_import_type(self, instance, row, default_action=RowResult.IMPORT_TYPE_SKIP):
        if instance is None:
            action = RowResult.IMPORT_TYPE_NEW
        elif isinstance(instance, self.Meta.model):
            action = RowResult.IMPORT_TYPE_UPDATE
        else:
            action = default_action
        return action

    def get_instance(self, row):
        teambition_id = row['teambition_id']
        try:
            # 没有处理 MultipleObjectsReturned 情况
            tb_object = self.Meta.model.objects.get(teambition_id=teambition_id)
            return tb_object
        except self.Meta.model.DoesNotExist:
            return None

    class Meta(BaseResource.Meta):
        serializer_class = TeamBitionSerializer
        model = None
